#!/opt/splunk/bin/python 

import logging, sys, ConfigParser, urllib, httplib2, json, os
from urlparse import urlparse


# Set up config
Config = ConfigParser.RawConfigParser()
Config.read( os.path.join( os.environ['SPLUNK_HOME'], 'bin', 'scripts', 'xMConfig.cfg' ) )

# Set up logging
LOGDIR = os.path.join( os.environ['SPLUNK_HOME'], 'var', 'log', 'splunk' )
logging.basicConfig( format = Config.get( 'Logging', 'format' ),
                     filename = os.path.join( LOGDIR, Config.get( 'Logging', 'filename' ) ),
                     level = Config.get( 'Logging', 'level' ) )

# Get the args. We'd rather get them from os.environ, but
# it is really nice for testing to be able to just pass them
# via the command line, so we'll check for that
# Per the docs:
# SPLUNK_ARG_0 Script name
# SPLUNK_ARG_1 Number of events returned
# SPLUNK_ARG_2 Search terms
# SPLUNK_ARG_3 Fully qualified query string
# SPLUNK_ARG_4 Name of report
# SPLUNK_ARG_5 Trigger reason (for example, "The number of events was greater than 1")
# SPLUNK_ARG_6 Browser URL to view the report
# SPLUNK_ARG_7 Not used for historical reasons

try:
  SPLUNK_ARG_1 = os.environ[ 'SPLUNK_ARG_1' ]
  SPLUNK_ARG_2 = os.environ[ 'SPLUNK_ARG_2' ]
  SPLUNK_ARG_3 = os.environ[ 'SPLUNK_ARG_3' ]
  SPLUNK_ARG_4 = os.environ[ 'SPLUNK_ARG_4' ]
  SPLUNK_ARG_5 = os.environ[ 'SPLUNK_ARG_5' ]
  SPLUNK_ARG_6 = os.environ[ 'SPLUNK_ARG_6' ]
  
except KeyError:
  SPLUNK_ARG_1 = sys.argv[1]
  SPLUNK_ARG_2 = sys.argv[2]
  SPLUNK_ARG_3 = sys.argv[3] 
  SPLUNK_ARG_4 = sys.argv[4] 
  SPLUNK_ARG_5 = sys.argv[5] 
  SPLUNK_ARG_6 = sys.argv[6] 

except Exception, e:
  logging.error( "Error setting args! Exception: '" + str( e ) + "'" )


args = [SPLUNK_ARG_1, SPLUNK_ARG_2, SPLUNK_ARG_3, SPLUNK_ARG_4, SPLUNK_ARG_5, SPLUNK_ARG_6 ]
logging.debug( "Args: " + str( args ) )
logging.info( "Alert name '" + SPLUNK_ARG_4 + "' fired" )

numEvents   = SPLUNK_ARG_1
searchTerms = SPLUNK_ARG_2
qryString   = SPLUNK_ARG_3
reportName  = SPLUNK_ARG_4
trigReason  = SPLUNK_ARG_5
reportURL   = SPLUNK_ARG_6

# Build payload
payload = json.dumps( {
  "properties": {
    "Number of Events": numEvents,
    "Search Terms": searchTerms,
    "Query String": qryString,
    "Report Name": reportName,
    "Trigger Reason": trigReason,
    "Report URL": reportURL,
    "xM URL": '{uri.scheme}://{uri.netloc}/'.format( uri=urlparse( Config.get( 'xM Creds', 'endpoint' ) ) ),
    
  },
  "recipients": []

})

logging.debug( "Payload: %s", str( payload ) )

# Build the authentication and content headers
# The xMatters web servers use ssl with a certificate from
# Starfield/Go Daddy. From time to time, a library doesn't
# recognize this as a valid authority, so we provide the
# cert to use. 
# For reference, this cert was downloaded from:
# https://certs.godaddy.com/repository/sf-class2-root.crt
CERT = os.path.join( os.environ['SPLUNK_HOME'], 'bin', 'scripts', 'sf-class2-root.crt' )
try:
  h = httplib2.Http(".cache", ca_certs=CERT )
  h.add_credentials( Config.get( 'xM Creds', 'username' ), Config.get( 'xM Creds', 'password' ) )
  headers = {'Content-type': 'application/json' }

  # Make the call
  resp, content = h.request( Config.get( 'xM Creds', 'endpoint' ), "POST", payload, headers )
except Exception, e:
  logging.error( "Error sending request. Exception: '" + str( e ) + "'" )
  sys.exit()

logging.debug( "Response: " + content )

contentjs = json.loads( content )
try:
  eventID = contentjs[ 'id' ]
  logging.info( "Successfully sent to xMatters. EventID: " + eventID )  
except KeyError:
  logging.error( "Error returned from xMatters: " + content )
